@@ -2,11 +2,12 @@ from .schemdraw import Drawing, use, config, theme
 from .segments import Segment, SegmentCircle, SegmentArc, SegmentText, SegmentPoly, SegmentBezier, SegmentArrow
 from .transform import Transform
 from .types import ImageFormat
+from .backends.svg import config as svgconfig
 from .backends.svg import settextmode
 
 __all__ = [
     "Drawing", "use", "config", "theme", "Segment", "SegmentCircle", "SegmentArc", "SegmentText",
-    "SegmentPoly", "SegmentBezier", "Transform", "ImageFormat", "settextmode"
+    "SegmentArrow", "SegmentPoly", "SegmentBezier", "Transform", "ImageFormat", "settextmode", "svgconfig"
 ]
 
-__version__ = '0.14'
+__version__ = '0.17'
