@@ -1,20 +1,23 @@
 ''' Signal processing elements '''
 
 import sys
+import math
+from typing import Sequence
+
 if sys.version_info >= (3, 8):
     from typing import Literal
 else:
     from typing_extensions import Literal
 
-import math
-from typing import Sequence
-
-from ..util import linspace
-from ..segments import Segment, SegmentCircle, SegmentText
+from ..util import linspace, Point
+from ..segments import Segment, SegmentCircle, SegmentText, SegmentArc
 from ..elements import Element
 from ..types import XY
 
 
+FilterType = Literal['lp', 'bp', 'hp', 'notch']
+
+
 class Square(Element):
     ''' Empty square element
 
@@ -64,8 +67,8 @@ class Circle(Element):
         self.anchors['W'] = (0, 0)
         self.anchors['NW'] = (rad-k, k)
         self.anchors['NE'] = (rad+k, k)
-        self.anchors['SW'] = (rad-k, k)
-        self.anchors['SE'] = (rad+k, k)
+        self.anchors['SW'] = (rad-k, -k)
+        self.anchors['SE'] = (rad+k, -k)
         self.anchors['center'] = (rad, 0)
 
 
@@ -129,8 +132,8 @@ class Mixer(Circle):
             * SW
             * SE
     '''
-    def __init__(self, *d, N: str=None, E: str=None, S: str=None, W: str=None,
-                 font: str=None, fontsize: float=10, **kwargs):
+    def __init__(self, *d, N: str = None, E: str = None, S: str = None, W: str = None,
+                 font: str = None, fontsize: float = 10, **kwargs):
         super().__init__(*d, **kwargs)
         rad = .5
         k = rad*math.sqrt(2)/2  # Diagonal distance
@@ -158,8 +161,8 @@ class Mixer(Circle):
 
 class Speaker(Element):
     ''' Speaker with only one terminal '''
-    def __init__(self, d=None, **kwargs):
-        super().__init__(d, **kwargs)
+    def __init__(self, *d, **kwargs):
+        super().__init__(*d, **kwargs)
         self.segments.append(Segment([(0, 0), (0, 0.25), (0.25, 0.25),
                                       (0.25, -.25), (0, -.25), (0, 0)]))
         self.segments.append(Segment([(0.25, 0.25), (0.5, 0.5), (0.5, -0.5),
@@ -180,7 +183,7 @@ class Amp(Element):
         self.segments.append(Segment([(0, 0), (0, -amph/2), (ampl, 0),
                                       (0, amph/2), (0, 0)]))
         self.params['drop'] = (ampl, 0)
-        self.anchors['in'] = (0, 0)
+        self.anchors['input'] = (0, 0)
         self.anchors['out'] = (ampl, 0)
 
 
@@ -239,11 +242,11 @@ class Filter(Square):
             * E
             * W
     '''
-    def __init__(self, *d, response: Literal['lp', 'bp', 'hp', 'notch']=None, **kwargs):
+    def __init__(self, *d, response: FilterType = None, **kwargs):
         super().__init__(*d, **kwargs)
         path = _makesine()
-        path1 = [[p[0], p[1]+.25] for p in path]
-        path2 = [[p[0], p[1]-.25] for p in path]
+        path1 = [Point((p[0], p[1]+.25)) for p in path]
+        path2 = [Point((p[0], p[1]-.25)) for p in path]
         self.segments.append(Segment(path))
         self.segments.append(Segment(path1))
         self.segments.append(Segment(path2))
@@ -276,7 +279,7 @@ class Adc(Element):
         self.params['lblloc'] = 'center'
         self.params['lblofst'] = 0
         self.params['drop'] = (1.4, 0)
-        self.anchors['in'] = (0, 0)
+        self.anchors['input'] = (0, 0)
         self.anchors['out'] = (1.4, 0)
         self.anchors['E'] = (1.4, 0)
         self.anchors['W'] = (0, 0)
@@ -298,7 +301,7 @@ class Dac(Element):
         self.params['lblloc'] = 'center'
         self.params['lblofst'] = 0
         self.params['drop'] = (1.4, 0)
-        self.anchors['in'] = (0, 0)
+        self.anchors['input'] = (0, 0)
         self.anchors['out'] = (1.4, 0)
         self.anchors['E'] = (1.4, 0)
         self.anchors['W'] = (0, 0)
@@ -319,3 +322,53 @@ class Demod(Square):
         self.segments.append(Segment([(.3, .25), (.7, 0), (.3, -.25), (.3, .25)]))
         self.segments.append(Segment([(.7, .25), (.7, -.25)]))
         self.segments.append(Segment([(.7, 0), (.85, 0)]))
+
+
+class Circulator(Circle):
+    ''' Circulator (circle with an arrow in it)
+
+        Anchors:
+            * N
+            * S
+            * E
+            * W
+    '''
+    def __init__(self, *d, **kwargs):
+        super().__init__(*d, **kwargs)
+        radius = 0.5
+        self.segments.append(
+            SegmentArc((0.5, 0), 0.9 * radius, 0.9 * radius, 20, 200, "cw"))
+
+
+class Isolator(Square):
+    ''' Isolator (box with an arrow in it)
+
+        Anchors:
+            * N
+            * S
+            * E
+            * W
+    '''
+    def __init__(self, *d, **kwargs):
+        super().__init__(*d, **kwargs)
+        margin = 0.25
+        path = ((margin, 0), (1 - margin, 0))
+        self.segments.append(Segment(path, arrow="->"))
+
+
+class VGA(Amp):
+    ''' Variable Gain Amplifier (amplifier symbol with an arrow over it)
+
+        Args:
+            tuneup: Set tune above or below the symbol
+
+        Anchors:
+            * input
+            * out
+            * tune
+    '''
+    def __init__(self, tuneup: bool = True, *d, **kwargs):
+        super().__init__(*d, **kwargs)
+        path = ((-0.1, -0.5), (0.75, 0.5))
+        self.segments.append(Segment(path, arrow="->"))
+        self.anchors['tune'] = (0.325, 0.5 if tuneup else -0.5)
\ No newline at end of file
